// Generated by CoffeeScript 1.12.5

/*!
--- atualiza ---

Released under MIT license.

by Gustavo Vargas - @xgvargas - 2017

Original coffee code and issues at: https://github.com/xgvargas/atualiza
 */
var Promise, Table, argv, colors, dependencies, e, exec, execAsync, iterativeTable, local_package, local_package_fn, path, ref, semver, tty, writeFileSync, yargs,
  hasProp = {}.hasOwnProperty;

path = require('path');

Promise = require('bluebird');

exec = require('child_process').exec;

writeFileSync = require('fs').writeFileSync;

tty = require('terminal-kit').terminal;

semver = require('semver');

Table = require('easy-table');

yargs = require('yargs');

colors = require('colors');

argv = yargs.version(require('../package.json').version).usage('Usage: $0 [options] [path]').help('help').alias('h', 'help').strict(true).options({
  'global': {
    type: 'boolean',
    alias: 'g',
    describe: 'Work with global npm packages'
  },
  'all': {
    type: 'boolean',
    alias: 'a',
    describe: 'List all installed packages'
  },
  'safe': {
    type: 'boolean',
    alias: 's',
    describe: 'Do NOT touch my package.json!'
  },
  'exact': {
    type: 'boolean',
    alias: 'e',
    describe: 'Use exact version instead of ^version'
  },
  'concurrency': {
    type: 'number',
    "default": 8,
    describe: 'Set the concurrencity number (1-50)'
  }
}).argv;

if (argv._.length > 1) {
  yargs.showHelp();
  tty.processExit(1);
}

if (!((1 <= (ref = argv.concurrency) && ref <= 50))) {
  yargs.showHelp();
  tty.processExit(1);
}

if (argv._.length === 1) {
  try {
    process.chdir(argv._[0]);
  } catch (error1) {
    tty.red('\nOops! Can´t change current directory to: `%s`\n', argv._[0]);
    tty.processExit(1);
  }
}

if (!argv.global) {
  try {
    local_package_fn = path.join(process.cwd(), 'package.json');
    local_package = require(local_package_fn);
  } catch (error1) {
    e = error1;
    tty.red('\nOops! Cant find a `package.json` file at `%s`\n', process.cwd());
    tty.processExit(1);
  }
  dependencies = Object.assign({}, local_package.dependencies);
  dependencies = Object.assign(dependencies, local_package.devDependencies || {});
}

iterativeTable = function(items) {
  return new Promise(function(resolve, reject) {
    var col, format, getVer, row, selecteds, showTable;
    col = -1;
    row = -1;
    selecteds = {};
    getVer = function(item, type) {
      if (argv.global) {
        if (semver.gt(item._newest, item.version)) {
          return item._newest;
        } else {
          return '';
        }
      } else {
        if (type === 'best') {
          if (semver.gt(item._best, item.version)) {
            return item._best;
          } else {
            return '';
          }
        }
        if (type === 'newest') {
          if (semver.gt(item._newest, item._best)) {
            return item._newest;
          } else {
            return '';
          }
        }
      }
    };
    format = function(table, r, c, title, val, name) {
      if (row === -1 && val) {
        row = r;
        col = c;
      }
      if (selecteds[name] === val && r === row && c === col) {
        val = colors.yellow('[ ') + colors.bgGreen.black(val) + colors.yellow(' ]');
      } else if (r === row && c === col) {
        val = colors.yellow('[ ') + val + colors.yellow(' ]');
      } else if (selecteds[name] === val) {
        val = '  ' + colors.bgGreen.black(val) + '  ';
      } else {
        val = '  ' + val + '  ';
      }
      return table.cell(title, val);
    };
    showTable = function() {
      var t;
      t = new Table;
      items.forEach(function(item, row) {
        var bestVal, newVal;
        t.cell('Package', item.name);
        t.cell('Current', item.version);
        if (argv.global) {
          newVal = getVer(item);
          format(t, row, 2, 'Newest', newVal, item.name);
        } else {
          bestVal = getVer(item, 'best');
          format(t, row, 2, 'Best', bestVal, item.name);
          newVal = getVer(item, 'newest');
          format(t, row, 3, 'Newest', newVal, item.name);
        }
        return t.newRow();
      });
      tty(t.toString());
      return tty.up(2 + items.length);
    };
    showTable();
    if (row === -1) {
      tty.down(2 + items.length);
      return resolve(null);
    }
    tty.grabInput(true);
    return tty.on('key', function(name, data) {
      var j, l, r, ref1, ref2, ref3;
      switch (name) {
        case 'UP':
          for (r = j = ref1 = row - 1; j >= 0; r = j += -1) {
            if (getVer(items[r], 'best')) {
              row = r;
              col = 2;
              break;
            }
            if (getVer(items[r], 'newest')) {
              row = r;
              col = 3;
              break;
            }
          }
          return showTable();
        case 'DOWN':
          for (r = l = ref2 = row + 1, ref3 = items.length; l < ref3; r = l += 1) {
            if (getVer(items[r], 'best')) {
              row = r;
              col = 2;
              break;
            }
            if (getVer(items[r], 'newest')) {
              row = r;
              col = 3;
              break;
            }
          }
          return showTable();
        case 'LEFT':
          if (!argv.global) {
            if (getVer(items[row], 'best') && getVer(items[row], 'newest')) {
              col = 2;
            }
            return showTable();
          }
          break;
        case 'RIGHT':
          if (!argv.global) {
            if (getVer(items[row], 'best') && getVer(items[row], 'newest')) {
              col = 3;
            }
            return showTable();
          }
          break;
        case ' ':
          if (argv.global) {
            if (selecteds[items[row].name] === items[row]._newest) {
              delete selecteds[items[row].name];
            } else {
              selecteds[items[row].name] = items[row]._newest;
            }
          } else {
            if (selecteds[items[row].name] === (col === 2 ? items[row]._best : items[row]._newest)) {
              delete selecteds[items[row].name];
            } else {
              selecteds[items[row].name] = col === 2 ? items[row]._best : items[row]._newest;
            }
          }
          return showTable();
        case 'ENTER':
          if (Object.keys(selecteds).length) {
            tty.grabInput(false);
            row = -2;
            showTable();
            tty.down(2 + items.length);
            return resolve(selecteds);
          }
          break;
        case 'ESCAPE':
        case 'CTRL_C':
        case 'q':
          tty.grabInput(false);
          row = -2;
          showTable();
          tty.down(2 + items.length);
          return resolve(null);
      }
    });
  });
};

execAsync = function(cmd, ops) {
  return new Promise(function(resolve, reject) {
    return exec(cmd, ops, function(err, stdout, stderr) {
      if (err) {
        return reject({
          stdout: stdout,
          stderr: stderr
        });
      }
      return resolve(stdout);
    });
  });
};

tty.hideCursor(true);

execAsync('npm ls --depth=0 --json' + (argv.global ? ' --global' : ''), {
  maxBuffer: 1024 * 1000
})["catch"](function(error) {
  return error.stdout;
}).then(function(all_packs) {
  var all_packs_deps, bar, p, packs, v;
  all_packs_deps = JSON.parse(all_packs).dependencies;
  packs = (function() {
    var results;
    results = [];
    for (p in all_packs_deps) {
      v = all_packs_deps[p];
      v.name = p;
      results.push(v);
    }
    return results;
  })();
  tty("\nDiscovered ^m%d^ packages. Getting their information...\n", packs.length);
  bar = tty.progressBar({
    items: packs.length,
    eta: true,
    percent: true,
    itemSize: 0,
    width: 50
  });
  return Promise.map(packs, function(item) {
    return execAsync("npm view " + item.name + " versions --json").then(function(_versions) {
      var versions;
      bar.itemDone();
      versions = JSON.parse(_versions);
      item._newest = versions[versions.length - 1];
      if (argv.global) {
        item._best = item._newest;
      } else {
        item._best = semver.maxSatisfying(versions, dependencies[item.name]);
      }
      return item;
    })["catch"](function(err) {
      tty('\n\nOOOOOps!\n');
      tty.red(err);
      return {};
    });
  }, {
    concurrency: argv.concurrency
  });
}).then(function(data) {
  var elegible, no_semver, valids;
  tty.eraseLine();
  no_semver = data.filter(function(i) {
    return !i._best;
  });
  valids = data.filter(function(i) {
    return i._best;
  });
  elegible = valids.filter(function(i) {
    try {
      return semver.gt(i._best, i.version) || semver.gt(i._newest, i.version);
    } catch (error1) {
      no_semver.push(i);
      valids.splice(valids.indexOf(i), 1);
      return false;
    }
  });
  if (no_semver.length) {
    tty.red('\nIgnored packages (not using semver):');
    no_semver.forEach(function(i) {
      return tty('\n%s @%s', i.name, i.version);
    });
  }
  if (elegible.length || argv.all) {
    tty('\n\n');
    return iterativeTable(argv.all ? valids : elegible);
  } else {
    tty.white('\n\nGreat!! Everything is up to date!\n');
    tty.hideCursor(false);
    return tty.processExit(0);
  }
}).then(function(update_info) {
  var cmd, err, k, ref1, ref2, ref3, v;
  if (!update_info) {
    return '';
  } else {
    if (!argv.global) {
      if (!argv.safe) {
        for (k in update_info) {
          if (!hasProp.call(update_info, k)) continue;
          v = update_info[k];
          if (!argv.exact) {
            v = '^' + v;
          }
          if ((ref1 = local_package.dependencies) != null ? ref1[k] : void 0) {
            local_package.dependencies[k] = v;
          }
          if ((ref2 = local_package.devDependencies) != null ? ref2[k] : void 0) {
            local_package.devDependencies[k] = v;
          }
        }
        try {
          writeFileSync(local_package_fn, JSON.stringify(local_package, null, 2) + '\n');
        } catch (error1) {
          err = error1;
          tty.red('\n\nOops! Can´t write to %s\n', local_package_fn);
          tty(err);
        }
      } else {
        tty.yellow('\n\nRespecting your decision to NOT change your `package.json`\n');
      }
    }
    if (argv.global && ((ref3 = process.platform) === 'linux')) {
      if (process.getuid() !== 0) {
        tty.red('\n\nTo update global packages please call `sudo atualiza -g`!\n');
        tty.hideCursor(false);
        tty.processExit(1);
      }
    }
    cmd = argv.global ? 'sudo npm i -g ' : 'npm i ';
    for (k in update_info) {
      if (!hasProp.call(update_info, k)) continue;
      v = update_info[k];
      cmd += k + "@" + v + " ";
    }
    tty.cyan('\n\nExecuting update...');
    tty.white('\n\nCommand: ');
    tty.green('%s\n\n', cmd);
    return execAsync(cmd, {
      stdio: [0, 1, 2]
    });
  }
}).then(function(output) {
  console.log(output);
  tty.hideCursor(false);
  return tty.processExit(0);
})["catch"](function(err) {
  tty('\n\nOops! Some bad thing just happened.... sorry about that!\n');
  tty.red(err);
  tty('\n\n');
  tty.hideCursor(false);
  return tty.processExit(1);
});
